@@ -33,10 +33,23 @@ extern "C" {
 #include <sys/types.h>
 
 void        g_mime_charset_map_init (void);
+void        g_mime_charset_map_shutdown (void);
 
-const char *g_mime_charset_locale_name (void);
+const char *g_mime_locale_charset (void);
+const char *g_mime_locale_language (void);
 
+const char *g_mime_charset_language (const char *charset);
+
+const char *g_mime_charset_canon_name (const char *charset);
+const char *g_mime_charset_iconv_name (const char *charset);
+
+#ifndef GMIME_DISABLE_DEPRECATED
 const char *g_mime_charset_name (const char *charset);
+const char *g_mime_charset_locale_name (void);
+#endif
+
+const char *g_mime_charset_iso_to_windows (const char *isocharset);
+
 
 typedef struct _GMimeCharset {
 	unsigned int mask;
@@ -51,6 +64,7 @@ const char *g_mime_charset_best_name (GMimeCharset *charset);
 
 const char *g_mime_charset_best (const char *in, size_t inlen);
 
+
 #ifdef __cplusplus
 }
 #endif /* __cplusplus */
