@@ -2,7 +2,7 @@
 /*
  *  Authors: Jeffrey Stedfast <fejj@ximian.com>
  *
- *  Copyright 2002 Ximian, Inc. (www.ximian.com)
+ *  Copyright 2002-2004 Ximian, Inc. (www.ximian.com)
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -51,8 +51,6 @@
 #define d(x) x
 #define _(x) x
 
-#define GMIME_ERROR_QUARK (g_quark_from_static_string ("gmime"))
-
 static void g_mime_gpg_context_class_init (GMimeGpgContextClass *klass);
 static void g_mime_gpg_context_init (GMimeGpgContext *ctx, GMimeGpgContextClass *klass);
 static void g_mime_gpg_context_finalize (GObject *object);
@@ -242,6 +240,7 @@ struct _GpgCtx {
 	unsigned char *statusptr;
 	unsigned int statusleft;
 	
+	char *need_id;
 	char *passwd;
 	
 	GMimeStream *istream;
@@ -310,6 +309,7 @@ gpg_ctx_new (GMimeSession *session, const char *path)
 	gpg->bad_passwds = 0;
 	gpg->need_passwd = FALSE;
 	gpg->send_passwd = FALSE;
+	gpg->need_id = NULL;
 	gpg->passwd = NULL;
 	
 	gpg->validsig = FALSE;
@@ -466,8 +466,12 @@ gpg_ctx_free (struct _GpgCtx *gpg)
 	
 	g_free (gpg->statusbuf);
 	
-	if (gpg->passwd)
+	g_free (gpg->need_id);
+	
+	if (gpg->passwd) {
+		memset (gpg->passwd, 0, strlen (gpg->passwd));
 		g_free (gpg->passwd);
+	}
 	
 	if (gpg->istream)
 		g_mime_stream_unref (gpg->istream);
@@ -612,6 +616,7 @@ gpg_ctx_op_start (struct _GpgCtx *gpg)
 	char *status_fd = NULL, *passwd_fd = NULL;
 	int i, maxfd, errnosave, fds[10];
 	GPtrArray *argv;
+	int flags;
 	
 	for (i = 0; i < 10; i++)
 		fds[i] = -1;
@@ -671,13 +676,21 @@ gpg_ctx_op_start (struct _GpgCtx *gpg)
 	if (gpg->need_passwd) {
 		close (fds[8]);
 		gpg->passwd_fd = fds[9];
-		fcntl (gpg->passwd_fd, F_SETFL, O_NONBLOCK);
+		flags = (flags = fcntl (gpg->passwd_fd, F_GETFL)) == -1 ? O_WRONLY : flags;
+		fcntl (gpg->passwd_fd, F_SETFL, flags | O_NONBLOCK);
 	}
 	
-	fcntl (gpg->stdin_fd, F_SETFL, O_NONBLOCK);
-	fcntl (gpg->stdout_fd, F_SETFL, O_NONBLOCK);
-	fcntl (gpg->stderr_fd, F_SETFL, O_NONBLOCK);
-	fcntl (gpg->status_fd, F_SETFL, O_NONBLOCK);
+	flags = (flags = fcntl (gpg->stdin_fd, F_GETFL)) == -1 ? O_WRONLY : flags;
+	fcntl (gpg->stdin_fd, F_SETFL, flags | O_NONBLOCK);
+	
+	flags = (flags = fcntl (gpg->stdout_fd, F_GETFL)) == -1 ? O_RDONLY : flags;
+	fcntl (gpg->stdout_fd, F_SETFL, flags | O_NONBLOCK);
+	
+	flags = (flags = fcntl (gpg->stderr_fd, F_GETFL)) == -1 ? O_RDONLY : flags;
+	fcntl (gpg->stderr_fd, F_SETFL, flags | O_NONBLOCK);
+	
+	flags = (flags = fcntl (gpg->status_fd, F_GETFL)) == -1 ? O_RDONLY : flags;
+	fcntl (gpg->status_fd, F_SETFL, flags | O_NONBLOCK);
 	
 	return 0;
 	
@@ -743,7 +756,7 @@ gpg_ctx_parse_status (struct _GpgCtx *gpg, GError **err)
 	d(printf ("status: %s\n", status));
 	
 	if (strncmp (status, "[GNUPG:] ", 9) != 0) {
-		g_set_error (err, GMIME_ERROR_QUARK, GMIME_ERROR_PARSE_ERROR,
+		g_set_error (err, GMIME_ERROR, GMIME_ERROR_PARSE_ERROR,
 			     _("Unexpected GnuPG status message encountered:\n\n%s"),
 			     status);
 		return -1;
@@ -758,7 +771,7 @@ gpg_ctx_parse_status (struct _GpgCtx *gpg, GError **err)
 		status += 12;
 		status = next_token (status, &hint);
 		if (!hint) {
-			g_set_error (err, GMIME_ERROR_QUARK, GMIME_ERROR_PARSE_ERROR,
+			g_set_error (err, GMIME_ERROR, GMIME_ERROR_PARSE_ERROR,
 				     _("Failed to parse gpg userid hint."));
 			return -1;
 		}
@@ -776,34 +789,35 @@ gpg_ctx_parse_status (struct _GpgCtx *gpg, GError **err)
 		
 		g_hash_table_insert (gpg->userid_hint, hint, user);
 	} else if (!strncmp (status, "NEED_PASSPHRASE ", 16)) {
-		char *prompt, *userid, *passwd;
-		const char *name;
+		char *userid;
 		
 		status += 16;
 		
 		status = next_token (status, &userid);
 		if (!userid) {
-			g_set_error (err, GMIME_ERROR_QUARK, GMIME_ERROR_PARSE_ERROR,
+			g_set_error (err, GMIME_ERROR, GMIME_ERROR_PARSE_ERROR,
 				     _("Failed to parse gpg passphrase request."));
 			return -1;
 		}
 		
-		name = g_hash_table_lookup (gpg->userid_hint, userid);
-		if (!name)
-			name = userid;
+		g_free (gpg->need_id);
+		gpg->need_id = userid;
+	} else if (!strncmp (status, "GET_HIDDEN passphrase.enter", 27)) {
+		char *prompt, *passwd;
+		const char *name;
+		
+		if (!(name = g_hash_table_lookup (gpg->userid_hint, gpg->need_id)))
+			name = gpg->userid;
 		
 		prompt = g_strdup_printf (_("You need a passphrase to unlock the key for\n"
 					    "user: \"%s\""), name);
 		
-		passwd = g_mime_session_request_passwd (gpg->session, prompt, TRUE, userid, err);
+		passwd = g_mime_session_request_passwd (gpg->session, prompt, TRUE, gpg->need_id, err);
 		g_free (prompt);
 		
-		g_free (gpg->userid);
-		gpg->userid = userid;
-		
 		if (passwd == NULL) {
 			if (err && *err == NULL)
-				g_set_error (err, GMIME_ERROR_QUARK, ECANCELED, _("Canceled."));
+				g_set_error (err, GMIME_ERROR, ECANCELED, _("Canceled."));
 			
 			return -1;
 		}
@@ -821,13 +835,13 @@ gpg_ctx_parse_status (struct _GpgCtx *gpg, GError **err)
 		g_mime_session_forget_passwd (gpg->session, gpg->userid, err);
 		
 		if (gpg->bad_passwds == 3) {
-			g_set_error (err, GMIME_ERROR_QUARK, GMIME_ERROR_BAD_PASSWORD,
+			g_set_error (err, GMIME_ERROR, GMIME_ERROR_BAD_PASSWORD,
 				     _("Failed to unlock secret key: 3 bad passphrases given."));
 			return -1;
 		}
 	} else if (!strncmp (status, "UNEXPECTED ", 11)) {
 		/* this is an error */
-		g_set_error (err, GMIME_ERROR_QUARK, GMIME_ERROR_GENERAL,
+		g_set_error (err, GMIME_ERROR, GMIME_ERROR_GENERAL,
 			     _("Unexpected response from GnuPG: %s"),
 			     status + 11);
 		return -1;
@@ -837,10 +851,10 @@ gpg_ctx_parse_status (struct _GpgCtx *gpg, GError **err)
 		
 		diagnostics = gpg_ctx_get_diagnostics (gpg);
 		if (diagnostics && *diagnostics)
-			g_set_error (err, GMIME_ERROR_QUARK, GMIME_ERROR_GENERAL,
+			g_set_error (err, GMIME_ERROR, GMIME_ERROR_GENERAL,
 				     diagnostics);
 		else
-			g_set_error (err, GMIME_ERROR_QUARK, GMIME_ERROR_GENERAL,
+			g_set_error (err, GMIME_ERROR, GMIME_ERROR_GENERAL,
 				     _("No data provided"));
 		
 		return -1;
@@ -878,7 +892,7 @@ gpg_ctx_parse_status (struct _GpgCtx *gpg, GError **err)
 			} else if (!strncmp (status, "END_ENCRYPTION", 14)) {
 				/* nothing to do, but we know the end is near? */
 			} else if (!strncmp (status, "NO_RECP", 7)) {
-				g_set_error (err, GMIME_ERROR_QUARK, GMIME_ERROR_NO_VALID_RECIPIENTS,
+				g_set_error (err, GMIME_ERROR, GMIME_ERROR_NO_VALID_RECIPIENTS,
 					     _("Failed to encrypt: No valid recipients specified."));
 				return -1;
 			}
@@ -1153,12 +1167,12 @@ gpg_ctx_op_step (struct _GpgCtx *gpg, GError **err)
 	errno = save;
 	
 	if (diagnostics && *diagnostics) {
-		g_set_error (err, GMIME_ERROR_QUARK, errno,
+		g_set_error (err, GMIME_ERROR, errno,
 			     _("Failed to %s via GnuPG: %s\n\n%s"),
 			     mode, g_strerror (errno),
 			     diagnostics);
 	} else {
-		g_set_error (err, GMIME_ERROR_QUARK, errno,
+		g_set_error (err, GMIME_ERROR, errno,
 			     _("Failed to %s vua GnuPG: %s\n"),
 			     mode, g_strerror (errno));
 	}
@@ -1271,7 +1285,7 @@ gpg_sign (GMimeCipherContext *context, const char *userid, GMimeCipherHash hash,
 	gpg_ctx_set_ostream (gpg, ostream);
 	
 	if (gpg_ctx_op_start (gpg) == -1) {
-		g_set_error (err, GMIME_ERROR_QUARK, errno,
+		g_set_error (err, GMIME_ERROR, errno,
 			     _("Failed to execute gpg: %s"),
 			     errno ? g_strerror (errno) : _("Unknown"));
 		gpg_ctx_free (gpg);
@@ -1296,7 +1310,7 @@ gpg_sign (GMimeCipherContext *context, const char *userid, GMimeCipherHash hash,
 		diagnostics = gpg_ctx_get_diagnostics (gpg);
 		errno = save;
 		
-		g_set_error (err, GMIME_ERROR_QUARK, errno, diagnostics);
+		g_set_error (err, GMIME_ERROR, errno, diagnostics);
 		gpg_ctx_free (gpg);
 		
 		return -1;
@@ -1357,7 +1371,7 @@ gpg_verify (GMimeCipherContext *context, GMimeCipherHash hash,
 		   the signature to a temp file. */
 		sigfile = swrite (sigstream);
 		if (sigfile == NULL) {
-			g_set_error (err, GMIME_ERROR_QUARK, errno,
+			g_set_error (err, GMIME_ERROR, errno,
 				     _("Cannot verify message signature: "
 				       "could not create temp file: %s"),
 				     g_strerror (errno));
@@ -1372,7 +1386,7 @@ gpg_verify (GMimeCipherContext *context, GMimeCipherHash hash,
 	gpg_ctx_set_istream (gpg, istream);
 	
 	if (gpg_ctx_op_start (gpg) == -1) {
-		g_set_error (err, GMIME_ERROR_QUARK, errno,
+		g_set_error (err, GMIME_ERROR, errno,
 			     _("Failed to execute gpg: %s"),
 			     errno ? g_strerror (errno) : _("Unknown"));
 		goto exception;
@@ -1434,7 +1448,7 @@ gpg_encrypt (GMimeCipherContext *context, gboolean sign, const char *userid,
 	}
 	
 	if (gpg_ctx_op_start (gpg) == -1) {
-		g_set_error (err, GMIME_ERROR_QUARK, errno,
+		g_set_error (err, GMIME_ERROR, errno,
 			     _("Failed to execute gpg: %s"),
 			     errno ? g_strerror (errno) : _("Unknown"));
 		gpg_ctx_free (gpg);
@@ -1459,7 +1473,7 @@ gpg_encrypt (GMimeCipherContext *context, gboolean sign, const char *userid,
 		diagnostics = gpg_ctx_get_diagnostics (gpg);
 		errno = save;
 		
-		g_set_error (err, GMIME_ERROR_QUARK, errno, diagnostics);
+		g_set_error (err, GMIME_ERROR, errno, diagnostics);
 		gpg_ctx_free (gpg);
 		
 		return -1;
@@ -1484,7 +1498,7 @@ gpg_decrypt (GMimeCipherContext *context, GMimeStream *istream,
 	gpg_ctx_set_ostream (gpg, ostream);
 	
 	if (gpg_ctx_op_start (gpg) == -1) {
-		g_set_error (err, GMIME_ERROR_QUARK, errno,
+		g_set_error (err, GMIME_ERROR, errno,
 			     _("Failed to execute gpg: %s"),
 			     errno ? g_strerror (errno) : _("Unknown"));
 		gpg_ctx_free (gpg);
@@ -1509,7 +1523,7 @@ gpg_decrypt (GMimeCipherContext *context, GMimeStream *istream,
 		diagnostics = gpg_ctx_get_diagnostics (gpg);
 		errno = save;
 		
-		g_set_error (err, GMIME_ERROR_QUARK, errno, diagnostics);
+		g_set_error (err, GMIME_ERROR, errno, diagnostics);
 		gpg_ctx_free (gpg);
 		
 		return -1;
@@ -1531,7 +1545,7 @@ gpg_import_keys (GMimeCipherContext *context, GMimeStream *istream, GError **err
 	gpg_ctx_set_istream (gpg, istream);
 	
 	if (gpg_ctx_op_start (gpg) == -1) {
-		g_set_error (err, GMIME_ERROR_QUARK, errno,
+		g_set_error (err, GMIME_ERROR, errno,
 			     _("Failed to execute gpg: %s"),
 			     errno ? g_strerror (errno) : _("Unknown"));
 		gpg_ctx_free (gpg);
@@ -1556,7 +1570,7 @@ gpg_import_keys (GMimeCipherContext *context, GMimeStream *istream, GError **err
 		diagnostics = gpg_ctx_get_diagnostics (gpg);
 		errno = save;
 		
-		g_set_error (err, GMIME_ERROR_QUARK, errno, diagnostics);
+		g_set_error (err, GMIME_ERROR, errno, diagnostics);
 		gpg_ctx_free (gpg);
 		
 		return -1;
@@ -1584,7 +1598,7 @@ gpg_export_keys (GMimeCipherContext *context, GPtrArray *keys, GMimeStream *ostr
 	}
 	
 	if (gpg_ctx_op_start (gpg) == -1) {
-		g_set_error (err, GMIME_ERROR_QUARK, errno,
+		g_set_error (err, GMIME_ERROR, errno,
 			     _("Failed to execute gpg: %s"),
 			     errno ? g_strerror (errno) : _("Unknown"));
 		gpg_ctx_free (gpg);
@@ -1609,7 +1623,7 @@ gpg_export_keys (GMimeCipherContext *context, GPtrArray *keys, GMimeStream *ostr
 		diagnostics = gpg_ctx_get_diagnostics (gpg);
 		errno = save;
 		
-		g_set_error (err, GMIME_ERROR_QUARK, errno, diagnostics);
+		g_set_error (err, GMIME_ERROR, errno, diagnostics);
 		gpg_ctx_free (gpg);
 		
 		return -1;
