@@ -1,9 +1,9 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
 /*
- *  Authors: Michael Zucchi <notzed@helixcode.com>
- *           Jeffrey Stedfast <fejj@helixcode.com>
+ *  Authors: Michael Zucchi <notzed@ximian.com>
+ *           Jeffrey Stedfast <fejj@ximian.com>
  *
- *  Copyright 2000 Helix Code, Inc. (www.helixcode.com)
+ *  Copyright 2000-2004 Ximian, Inc. (www.ximian.com)
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -52,13 +52,18 @@
 #include "gmime-iconv-utils.h"
 #include "gmime-host-utils.h"
 
-#define d(x)
-#define w(x) x
-
 #ifndef HAVE_ISBLANK
 #define isblank(c) (c == ' ' || c == '\t')
 #endif
 
+#ifdef ENABLE_WARNINGS
+#define w(x) x
+#else
+#define w(x)
+#endif /* ENABLE_WARNINGS */
+
+#define d(x)
+
 #define GMIME_UUENCODE_CHAR(c) ((c) ? (c) + ' ' : '`')
 #define	GMIME_UUDECODE_CHAR(c) (((c) - ' ') & 077)
 
@@ -785,19 +790,15 @@ g_mime_utils_generate_message_id (const char *fqdn)
 #ifdef HAVE_GETHOSTNAME
 		if (gethostname (host, sizeof (host)) == 0) {
 #ifdef HAVE_GETHOSTBYNAME
-			int ret, buflen = 1024, herr = 0;
+			size_t buflen = 1024;
+			int retval;
 			
-			do {
-				if (buf != NULL)
-					buf = g_realloc (buf, buflen);
-				else
-					buf = g_malloc (buflen);
-				
-				if ((ret = g_gethostbyname_r (host, &hostbuf, buf, buflen, &herr)) == ERANGE)
-					buflen += 256;
-			} while (ret != 0 && buflen < 8192);
+			buf = g_malloc (buflen);
 			
-			if (ret != 0) {
+			while ((retval = g_gethostbyname_r (host, &hostbuf, buf, buflen, NULL)) == ERANGE && buflen < 8192)
+				buf = g_realloc (buf, (buflen += 1024));
+			
+			if (retval == -1) {
 				g_free (buf);
 				buf = NULL;
 			}
