@@ -1,9 +1,9 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
 /*
- *  Authors: Michael Zucchi <notzed@helixcode.com>
- *           Jeffrey Stedfast <fejj@helixcode.com>
+ *  Authors: Michael Zucchi <notzed@ximian.com>
+ *           Jeffrey Stedfast <fejj@ximian.com>
  *
- *  Copyright 2000 Helix Code, Inc. (www.helixcode.com)
+ *  Copyright 2000-2004 Ximian, Inc. (www.ximian.com)
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -27,7 +27,7 @@
 #ifdef __cplusplus
 extern "C" {
 #pragma }
-#endif /* __cplusplus }*/
+#endif /* __cplusplus */
 
 #include <glib.h>
 #include <time.h>
@@ -83,9 +83,17 @@ gboolean g_mime_utils_text_is_8bit (const unsigned char *text, size_t len);
 GMimePartEncodingType g_mime_utils_best_encoding (const unsigned char *text, size_t len);
 
 /* utilities to (de/en)code headers */
+char *g_mime_utils_header_decode_text (const unsigned char *in);
+char *g_mime_utils_header_encode_text (const unsigned char *in);
+
+char *g_mime_utils_header_decode_phrase (const unsigned char *in);
+char *g_mime_utils_header_encode_phrase (const unsigned char *in);
+
+#ifndef GMIME_DISABLE_DEPRECATED
 char *g_mime_utils_8bit_header_decode (const unsigned char *in);
 char *g_mime_utils_8bit_header_encode (const unsigned char *in);
 char *g_mime_utils_8bit_header_encode_phrase (const unsigned char *in);
+#endif
 
 /* do incremental base64 (de/en)coding */
 size_t g_mime_utils_base64_decode_step (const unsigned char *in, size_t inlen, unsigned char *out, int *state, guint32 *save);
