@@ -144,7 +144,7 @@ internet_address_new_name (const char *name, const char *addr)
 	ia = internet_address_new ();
 	ia->type = INTERNET_ADDRESS_NAME;
 	if (name) {
-		ia->name = g_mime_utils_8bit_header_decode (name);
+		ia->name = g_mime_utils_header_decode_phrase (name);
 		g_mime_utils_unquote_string (ia->name);
 	}
 	ia->value.addr = g_strdup (addr);
@@ -169,7 +169,7 @@ internet_address_new_group (const char *name)
 	ia = internet_address_new ();
 	ia->type = INTERNET_ADDRESS_GROUP;
 	if (name) {
-		ia->name = g_mime_utils_8bit_header_decode (name);
+		ia->name = g_mime_utils_header_decode_phrase (name);
 		g_mime_utils_unquote_string (ia->name);
 	}
 	
@@ -191,7 +191,7 @@ internet_address_set_name (InternetAddress *ia, const char *name)
 	
 	g_free (ia->name);
 	if (name) {
-		ia->name = g_mime_utils_8bit_header_decode (name);
+		ia->name = g_mime_utils_header_decode_phrase (name);
 		g_mime_utils_unquote_string (ia->name);
 	} else
 		ia->name = NULL;
@@ -370,9 +370,9 @@ internet_address_list_concat (InternetAddressList *a, InternetAddressList *b)
  * Returns the number of internet addresses in @list.
  **/
 int
-internet_address_list_length (InternetAddressList *list)
+internet_address_list_length (const InternetAddressList *list)
 {
-	InternetAddressList *node;
+	const InternetAddressList *node;
 	int len = 0;
 	
 	node = list;
@@ -413,8 +413,8 @@ encoded_name (const char *raw, gboolean rfc2047_encode)
 	
 	g_return_val_if_fail (raw != NULL, NULL);
 	
-	if (rfc2047_encode && g_mime_utils_text_is_8bit (raw, strlen (raw))) {
-		name = g_mime_utils_8bit_header_encode_phrase (raw);
+	if (rfc2047_encode) {
+		name = g_mime_utils_header_encode_phrase (raw);
 	} else {
 		name = g_mime_utils_quote_string (raw);
 	}
@@ -423,7 +423,7 @@ encoded_name (const char *raw, gboolean rfc2047_encode)
 }
 
 static void
-internet_address_list_to_string_internal (InternetAddressList *list, gboolean encode, GString *string)
+internet_address_list_to_string_internal (const InternetAddressList *list, gboolean encode, GString *string)
 {
 	while (list) {
 		char *addr;
@@ -453,12 +453,12 @@ internet_address_list_to_string_internal (InternetAddressList *list, gboolean en
  * rfc822 format.
  **/
 char *
-internet_address_to_string (InternetAddress *ia, gboolean encode)
+internet_address_to_string (const InternetAddress *ia, gboolean encode)
 {
 	char *str = NULL;
 	
 	if (ia->type == INTERNET_ADDRESS_NAME) {
-		if (ia->name && *ia->name) {
+		if (ia->name) {
 			char *name;
 			
 			name = encoded_name (ia->name, encode);
@@ -497,7 +497,7 @@ internet_address_to_string (InternetAddress *ia, gboolean encode)
  * Returns a string containing the list of addresses in rfc822 format.
  **/
 char *
-internet_address_list_to_string (InternetAddressList *list, gboolean encode)
+internet_address_list_to_string (const InternetAddressList *list, gboolean encode)
 {
 	GString *string;
 	char *str;
